"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isId = isId;
exports.isElementId = isElementId;
exports.isNullableId = isNullableId;
exports.isParentedId = isParentedId;
exports.isParentedMap = isParentedMap;
exports.isEntitiesMap = isEntitiesMap;
exports.isSubEntityPathElement = isSubEntityPathElement;
exports.isSchemaReference = isSchemaReference;
exports.isIParentedId = isIParentedId;
exports.isParentedMutableMap = isParentedMutableMap;
exports.isMutableEntitiesMap = isMutableEntitiesMap;
exports.hasMappedElement = hasMappedElement;
exports.mappedElement = mappedElement;
exports.generateNewId = void 0;

var _uuid = require("uuid");

var _LazyMap = require("./LazyMap");

function isId(obj) {
  return typeof obj === 'number' || obj !== '' && typeof obj === 'string';
}

function isElementId(obj) {
  return typeof obj === 'object' && isId(obj._id) && typeof obj.__typename === 'string';
}

function isNullableId(obj) {
  return obj === null || isId(obj);
}

function isParentedId(obj) {
  return typeof obj === 'object' && isId(obj._id) && typeof obj.__typename === 'string' && (obj.parentType === null || typeof obj.parentType === 'string') && (obj.parentId === null || isId(obj.parentId));
}

function isParentedMap(obj) {
  if (obj && obj instanceof Map) {
    if (obj.size === 0) return true;

    for (const [key, val] of obj.entries()) {
      if (!(isId(key) && isParentedId(val))) {
        return false;
      }
    }

    return true;
  }

  return false;
}

function isEntitiesMap(obj) {
  if (typeof obj !== 'object') return false;
  let i = 0;

  for (const entityType of Object.keys(obj)) {
    if (!isParentedMap(obj[entityType])) {
      return false;
    }

    i++;
  }

  return i > 0;
}

function isSubEntityPathElement(obj) {
  if (typeof obj === 'string') {
    return true;
  }

  if (Array.isArray(obj) && obj.length === 2 && typeof obj[0] === 'string' && typeof obj[1] === 'number') {
    return true;
  }

  return false;
}

function isSchemaReference(obj) {
  return Boolean(obj && typeof obj === 'object' && '__schemaType' in obj && obj.__schemaType && typeof obj.__schemaType === 'string');
}

function isIParentedId(obj) {
  return obj && typeof obj === 'object' && obj._id !== undefined && isId(obj._id) && (typeof obj.parentId === 'undefined' || isId(obj.parentId) || obj.parentId === null);
}
/**
 * Checks if the object passed as parameter is a mutable
 * lazy map with Ids as keys and IParentedId subtypes as
 * values.
 *
 * @param obj
 * @returns {obj is ILazyMutableMap<Id, IParentedId>}
 */


function isParentedMutableMap(obj) {
  if (obj && obj instanceof _LazyMap.LazyMutableMap) {
    if (obj.getMap().size === 0) return true;

    for (const [key, val] of obj.getMap().entries()) {
      if (!(isId(key) && isParentedId(val))) {
        return false;
      }
    }

    return true;
  }

  return false;
}

function isMutableEntitiesMap(obj) {
  if (typeof obj !== 'object') return false;
  const fieldNames = Object.keys(obj);
  if (fieldNames.length < 1) return false;

  for (const fieldName of fieldNames) {
    if (!isParentedMutableMap(obj[fieldName])) {
      return false;
    }
  }

  return true;
}
/**
 * Checks if an element of type elementType with id elementId is in the document
 * hierarchy.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} docOrMaps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {boolean}
 */


function hasMappedElement(docOrMaps, elementType, elementId) {
  let maps;

  if (isEntitiesMap(docOrMaps) || isMutableEntitiesMap(docOrMaps)) {
    maps = docOrMaps;
  } else if (isEntitiesMap(docOrMaps.maps) || isMutableEntitiesMap(docOrMaps.maps)) {
    maps = docOrMaps.maps;
  } else {
    return false;
  }

  if (!(elementType in maps)) {
    return false;
  }

  const typeMap = maps[elementType];
  return typeMap.has(elementId);
}
/**
 * Given a map of entities or mutable entities, returns the element that corresponds to the type
 * and id provided as parameter.
 *
 * If the type requested is not mapped, a type error is thrown. If no element with the requested _id is found,
 * a reference error is thrown.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} maps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {EntitiesMaps<MapsInterface>[U] extends Map<Id, infer T> ? T : never}
 */


function mappedElement(docOrMaps, elementType, elementId) {
  let maps;

  if (isEntitiesMap(docOrMaps) || isMutableEntitiesMap(docOrMaps)) {
    maps = docOrMaps;
  } else if (isEntitiesMap(docOrMaps.maps) || isMutableEntitiesMap(docOrMaps.maps)) {
    maps = docOrMaps.maps;
  } else {
    maps = {};
  }

  if (!(elementType in maps)) {
    throw new TypeError(`Element type ${elementType} not found`);
  }

  const typeMap = maps[elementType];

  if (!typeMap.has(elementId)) {
    throw new ReferenceError(`Referential integrity: element ${elementType}.${elementId} not found`);
  }

  return typeMap.get(elementId);
}
/**
 * Generates a unique id to assign to an element.
 *
 * @returns {Id}
 */


const generateNewId = () => (0, _uuid.v1)();

exports.generateNewId = generateNewId;
//# sourceMappingURL=HUtils.js.map